import { defineConfig, isFunction } from '@agile-ts/utils';
import { logCodeManager } from '../../logCodeManager.js';
import { ComponentSubscriptionContainer } from './container/ComponentSubscriptionContainer.js';
import { CallbackSubscriptionContainer } from './container/CallbackSubscriptionContainer.js';

class SubController {
  constructor(agileInstance) {
    this.componentSubs = new Set();
    this.callbackSubs = new Set();
    this.mountedComponents = new Set();
    this.agileInstance = () => agileInstance;
  }
  subscribe(integrationInstance, subs, config = {}) {
    config = defineConfig(config, {
      waitForMount: this.agileInstance().config.waitForMount
    });
    const subscriptionContainer = this[isFunction(integrationInstance) ? "createCallbackSubscriptionContainer" : "createComponentSubscriptionContainer"](integrationInstance, subs, config);
    if (subscriptionContainer.isObjectBased && !Array.isArray(subs)) {
      const props = {};
      for (const key in subs)
        if (subs[key]["value"])
          props[key] = subs[key]["value"];
      return { subscriptionContainer, props };
    }
    return subscriptionContainer;
  }
  unsubscribe(subscriptionInstance) {
    const unsub = (subscriptionContainer) => {
      subscriptionContainer.ready = false;
      subscriptionContainer.subscribers.forEach((observer) => {
        subscriptionContainer.removeSubscription(observer);
      });
    };
    if (subscriptionInstance instanceof CallbackSubscriptionContainer) {
      unsub(subscriptionInstance);
      this.callbackSubs.delete(subscriptionInstance);
      if (process.env.NODE_ENV !== "production") {
        logCodeManager.log("15:01:00", { tags: ["subscription"] }, subscriptionInstance);
      }
      return;
    }
    if (subscriptionInstance instanceof ComponentSubscriptionContainer) {
      unsub(subscriptionInstance);
      this.componentSubs.delete(subscriptionInstance);
      if (process.env.NODE_ENV !== "production") {
        logCodeManager.log("15:01:01", { tags: ["subscription"] }, subscriptionInstance);
      }
      return;
    }
    if (subscriptionInstance["componentSubscriptionContainers"] !== null && Array.isArray(subscriptionInstance.componentSubscriptionContainers)) {
      subscriptionInstance.componentSubscriptionContainers.forEach((subContainer) => {
        unsub(subContainer);
        this.componentSubs.delete(subContainer);
        if (process.env.NODE_ENV !== "production") {
          logCodeManager.log("15:01:01", { tags: ["subscription"] }, subscriptionInstance);
        }
      });
      return;
    }
  }
  createComponentSubscriptionContainer(componentInstance, subs, config = {}) {
    const componentSubscriptionContainer = new ComponentSubscriptionContainer(componentInstance, subs, config);
    this.componentSubs.add(componentSubscriptionContainer);
    if (config.waitForMount) {
      if (this.mountedComponents.has(componentInstance))
        componentSubscriptionContainer.ready = true;
    } else
      componentSubscriptionContainer.ready = true;
    if (Array.isArray(componentInstance.componentSubscriptionContainers))
      componentInstance.componentSubscriptionContainers.push(componentSubscriptionContainer);
    else
      componentInstance["componentSubscriptionContainers"] = [
        componentSubscriptionContainer
      ];
    if (process.env.NODE_ENV !== "production") {
      logCodeManager.log("15:01:02", { tags: ["subscription"] }, componentSubscriptionContainer);
    }
    return componentSubscriptionContainer;
  }
  createCallbackSubscriptionContainer(callbackFunction, subs, config = {}) {
    const callbackSubscriptionContainer = new CallbackSubscriptionContainer(callbackFunction, subs, config);
    this.callbackSubs.add(callbackSubscriptionContainer);
    callbackSubscriptionContainer.ready = true;
    if (process.env.NODE_ENV !== "production") {
      logCodeManager.log("15:01:03", { tags: ["subscription"] }, callbackSubscriptionContainer);
    }
    return callbackSubscriptionContainer;
  }
  mount(componentInstance) {
    if (componentInstance["componentSubscriptionContainers"] != null && Array.isArray(componentInstance.componentSubscriptionContainers))
      componentInstance.componentSubscriptionContainers.map((c) => c.ready = true);
    this.mountedComponents.add(componentInstance);
  }
  unmount(componentInstance) {
    if (componentInstance["componentSubscriptionContainers"] != null && Array.isArray(componentInstance.componentSubscriptionContainers))
      componentInstance.componentSubscriptionContainers.map((c) => c.ready = false);
    this.mountedComponents.delete(componentInstance);
  }
}

export { SubController };
