import { defineConfig, isValidObject, flatMerge, isFunction, generateId, equal, notEqual } from '@agile-ts/utils';
import { logCodeManager } from '../logCodeManager.js';
import { State } from './state.js';
import { StatePersistent } from './state.persistent.js';

class EnhancedState extends State {
  constructor(agileInstance, initialValue, config = {}) {
    super(agileInstance, initialValue, config);
    this.isPersisted = false;
    this.computeExistsMethod = (v) => {
      return v != null;
    };
  }
  setKey(value) {
    var _a, _b;
    const oldKey = this._key;
    super.setKey(value);
    if (value != null && ((_a = this.persistent) == null ? void 0 : _a._key) === oldKey)
      (_b = this.persistent) == null ? void 0 : _b.setKey(value);
    return this;
  }
  undo(config = {}) {
    this.set(this.previousStateValue, config);
    return this;
  }
  reset(config = {}) {
    this.set(this.initialStateValue, config);
    return this;
  }
  patch(targetWithChanges, config = {}) {
    config = defineConfig(config, {
      addNewProperties: true
    });
    if (!isValidObject(this.nextStateValue, true)) {
      logCodeManager.log("14:03:02");
      return this;
    }
    if (!isValidObject(targetWithChanges, true)) {
      logCodeManager.log("00:03:01", {
        replacers: ["TargetWithChanges", "object"]
      });
      return this;
    }
    if (Array.isArray(targetWithChanges) && Array.isArray(this.nextStateValue)) {
      this.nextStateValue = [
        ...this.nextStateValue,
        ...targetWithChanges
      ];
    } else {
      this.nextStateValue = flatMerge(this.nextStateValue, targetWithChanges, { addNewProperties: config.addNewProperties });
    }
    this.ingest(config);
    return this;
  }
  watch(keyOrCallback, callback) {
    const generateKey = isFunction(keyOrCallback);
    let _callback;
    let key;
    if (generateKey) {
      key = generateId();
      _callback = keyOrCallback;
    } else {
      key = keyOrCallback;
      _callback = callback;
    }
    if (!isFunction(_callback)) {
      logCodeManager.log("00:03:01", {
        replacers: ["Watcher Callback", "function"]
      });
      return this;
    }
    this.addSideEffect(key, (instance) => {
      _callback(instance.value, key);
    }, { weight: 0 });
    return generateKey ? key : this;
  }
  removeWatcher(key) {
    this.removeSideEffect(key);
    return this;
  }
  onInaugurated(callback) {
    const watcherKey = "InauguratedWatcherKey";
    this.watch(watcherKey, (value, key) => {
      callback(value, key);
      this.removeSideEffect(watcherKey);
    });
    return this;
  }
  interval(handler, delay) {
    if (!isFunction(handler)) {
      logCodeManager.log("00:03:01", {
        replacers: ["Interval Callback", "function"]
      });
      return this;
    }
    if (this.currentInterval) {
      logCodeManager.log("14:03:03", { replacers: [] }, this.currentInterval);
      return this;
    }
    this.currentInterval = setInterval(() => {
      this.set(handler(this._value));
    }, delay != null ? delay : 1e3);
    return this;
  }
  clearInterval() {
    if (this.currentInterval) {
      clearInterval(this.currentInterval);
      delete this.currentInterval;
    }
  }
  get exists() {
    return !this.isPlaceholder && this.computeExistsMethod(this.value);
  }
  computeExists(method) {
    if (!isFunction(method)) {
      logCodeManager.log("00:03:01", {
        replacers: ["Compute Exists Method", "function"]
      });
      return this;
    }
    this.computeExistsMethod = method;
    return this;
  }
  computeValue(method) {
    if (!isFunction(method)) {
      logCodeManager.log("00:03:01", {
        replacers: ["Compute Value Method", "function"]
      });
      return this;
    }
    this.computeValueMethod = method;
    this.set(this.nextStateValue);
    return this;
  }
  is(value) {
    return equal(value, this.value);
  }
  isNot(value) {
    return notEqual(value, this.value);
  }
  invert() {
    switch (typeof this.nextStateValue) {
      case "boolean":
        this.set(!this.nextStateValue);
        break;
      case "object":
        if (Array.isArray(this.nextStateValue))
          this.set(this.nextStateValue.reverse());
        break;
      case "string":
        this.set(this.nextStateValue.split("").reverse().join(""));
        break;
      case "number":
        this.set(this.nextStateValue * -1);
        break;
      default:
        logCodeManager.log("14:03:04", {
          replacers: [typeof this.nextStateValue]
        });
    }
    return this;
  }
  persist(config = {}) {
    config = defineConfig(config, {
      key: this._key
    });
    if (this.persistent != null && this.isPersisted)
      return this;
    this.persistent = new StatePersistent(this, config);
    return this;
  }
  onLoad(callback) {
    if (!this.persistent)
      return this;
    if (!isFunction(callback)) {
      logCodeManager.log("00:03:01", {
        replacers: ["OnLoad Callback", "function"]
      });
      return this;
    }
    this.persistent.onLoad = callback;
    if (this.isPersisted)
      callback(true);
    return this;
  }
  getPersistableValue() {
    return this._value;
  }
}

export { EnhancedState };
