import { defineConfig, copy, generateId, equal, notEqual, createArrayFromObject, isFunction } from '@agile-ts/utils';
import { StateRuntimeJob } from './state.runtime.job.js';
import { logCodeManager } from '../logCodeManager.js';
import { Observer } from '../runtime/observer.js';

class StateObserver extends Observer {
  constructor(state, config = {}) {
    super(state.agileInstance(), defineConfig(config, {
      value: copy(state._value)
    }));
    this.state = () => state;
    this.nextStateValue = copy(state._value);
  }
  ingest(config = {}) {
    const state = this.state();
    if (state.isComputed) {
      state.compute().then((result) => {
        this.ingestValue(result, config);
      });
    } else {
      this.ingestValue(state.nextStateValue, config);
    }
  }
  ingestValue(newStateValue, config = {}) {
    const state = this.state();
    config = defineConfig(config, {
      perform: true,
      force: false,
      key: logCodeManager.allowLogging ? `${this.key != null ? this.key + "_" : ""}${generateId()}_value` : void 0
    });
    if (state.isPlaceholder) {
      config.force = true;
      config.overwrite = true;
    }
    this.nextStateValue = state.computeValueMethod ? copy(state.computeValueMethod(newStateValue)) : copy(newStateValue);
    if (equal(state._value, this.nextStateValue) && !config.force)
      return;
    const job = new StateRuntimeJob(this, config);
    this.agileInstance().runtime.ingest(job, {
      perform: config.perform
    });
  }
  perform(job) {
    const observer = job.observer;
    const state = observer.state();
    state.previousStateValue = copy(state._value);
    state._value = copy(observer.nextStateValue);
    state.nextStateValue = copy(observer.nextStateValue);
    if (job.config.overwrite) {
      state.initialStateValue = copy(state._value);
      state.previousStateValue = copy(state._value);
      state.isPlaceholder = false;
    }
    state.isSet = notEqual(state._value, state.initialStateValue);
    this.sideEffects(job);
    job.observer.previousValue = Object.freeze(copy(observer.value));
    job.observer.value = copy(state._value);
  }
  sideEffects(job) {
    var _a, _b, _c;
    const state = job.observer.state();
    if (Object.keys(state.sideEffects).length === 0)
      return;
    if ((_b = (_a = job.config) == null ? void 0 : _a.sideEffects) == null ? void 0 : _b.enabled) {
      const sideEffectArray = createArrayFromObject(state.sideEffects);
      sideEffectArray.sort(function(a, b) {
        return b.instance.weight - a.instance.weight;
      });
      for (const sideEffect of sideEffectArray) {
        if (isFunction(sideEffect.instance.callback)) {
          if (!((_c = job.config.sideEffects.exclude) == null ? void 0 : _c.includes(sideEffect.key)))
            sideEffect.instance.callback(job.observer.state(), job.config);
        }
      }
    }
  }
}

export { StateObserver };
