import { defineConfig, notEqual } from '@agile-ts/utils';
import { logCodeManager } from '../logCodeManager.js';
import { Storage } from './storage.js';

class Storages {
  constructor(agileInstance, config = {}) {
    this.storages = {};
    this.persistentInstances = {};
    this.agileInstance = () => agileInstance;
    config = defineConfig(config, {
      localStorage: false,
      defaultStorageKey: null
    });
    this.config = { defaultStorageKey: config.defaultStorageKey };
    if (config.localStorage)
      this.instantiateLocalStorage();
  }
  instantiateLocalStorage() {
    if (!Storages.localStorageAvailable()) {
      logCodeManager.log("11:02:00");
      return false;
    }
    const _localStorage = new Storage({
      key: "localStorage",
      async: false,
      methods: {
        get: localStorage.getItem.bind(localStorage),
        set: localStorage.setItem.bind(localStorage),
        remove: localStorage.removeItem.bind(localStorage)
      }
    });
    return this.register(_localStorage, { default: true });
  }
  register(storage, config = {}) {
    const hasRegisteredAnyStorage = notEqual(this.storages, {});
    if (Object.prototype.hasOwnProperty.call(this.storages, storage.key)) {
      logCodeManager.log("11:03:00", { replacers: [storage.key] });
      return false;
    }
    if (!hasRegisteredAnyStorage && config.default === false)
      logCodeManager.log("11:02:01");
    if (!hasRegisteredAnyStorage)
      config.default = true;
    this.storages[storage.key] = storage;
    if (config.default)
      this.config.defaultStorageKey = storage.key;
    for (const persistentKey of Object.keys(this.persistentInstances)) {
      const persistent = this.persistentInstances[persistentKey];
      if (persistent == null)
        continue;
      if (persistent.storageKeys.includes(storage.key)) {
        const isValid = persistent.validatePersistent();
        if (isValid)
          persistent.initialLoading();
        continue;
      }
      if (persistent.config.defaultStorageKey == null) {
        persistent.assignStorageKeys();
        const isValid = persistent.validatePersistent();
        if (isValid)
          persistent.initialLoading();
      }
    }
    logCodeManager.log("13:00:00", { replacers: [storage.key] }, storage);
    return true;
  }
  getStorage(storageKey) {
    if (!storageKey)
      return void 0;
    const storage = this.storages[storageKey];
    if (!storage) {
      logCodeManager.log("11:03:01", { replacers: [storageKey] });
      return void 0;
    }
    if (!storage.ready) {
      logCodeManager.log("11:03:02", { replacers: [storageKey] });
      return void 0;
    }
    return storage;
  }
  get(storageItemKey, storageKey) {
    if (!this.hasStorage()) {
      logCodeManager.log("11:03:03");
      return Promise.resolve(void 0);
    }
    if (storageKey) {
      const storage = this.getStorage(storageKey);
      if (storage)
        return storage.get(storageItemKey);
    }
    const defaultStorage = this.getStorage(this.config.defaultStorageKey);
    return (defaultStorage == null ? void 0 : defaultStorage.get(storageItemKey)) || Promise.resolve(void 0);
  }
  set(storageItemKey, value, storageKeys) {
    var _a;
    if (!this.hasStorage()) {
      logCodeManager.log("11:03:04");
      return;
    }
    if (storageKeys != null) {
      for (const storageKey of storageKeys)
        (_a = this.getStorage(storageKey)) == null ? void 0 : _a.set(storageItemKey, value);
      return;
    }
    const defaultStorage = this.getStorage(this.config.defaultStorageKey);
    defaultStorage == null ? void 0 : defaultStorage.set(storageItemKey, value);
  }
  remove(storageItemKey, storageKeys) {
    var _a;
    if (!this.hasStorage()) {
      logCodeManager.log("11:03:05");
      return;
    }
    if (storageKeys) {
      for (const storageKey of storageKeys)
        (_a = this.getStorage(storageKey)) == null ? void 0 : _a.remove(storageItemKey);
      return;
    }
    const defaultStorage = this.getStorage(this.config.defaultStorageKey);
    defaultStorage == null ? void 0 : defaultStorage.remove(storageItemKey);
  }
  hasStorage() {
    return notEqual(this.storages, {});
  }
  static localStorageAvailable() {
    try {
      localStorage.setItem("_myDummyKey_", "myDummyValue");
      localStorage.removeItem("_myDummyKey_");
      return true;
    } catch (e) {
      return false;
    }
  }
}

export { Storages };
